@@ -6,12 +6,10 @@
 
 // Zlib decompressor
 #include <zlib.h>
-#include <boost/log/trivial.hpp>
-#include <bson/bson.h>
+#include <bson.h>
 #include <sstream>
 #include <map>
-#include <boost/tokenizer.hpp>
-#include <boost/token_functions.hpp>
+#include <spdlog/spdlog.h>
 
 #include "ChunkMetric.h"
 #include "ConstDataRangeCursor.h"
@@ -27,18 +25,19 @@ write_log(const std::vector<std::string>  pref, const char *key, const bson_t *d
 #endif
 }
 
-Chunk::Chunk (const uint8_t *data, size_t size, int64_t id=-1, bool logMetricNames=0) {
+Chunk::Chunk (const uint8_t *data, size_t size, int64_t id=-1) : deltasInChunk(0), metricsInChunk(0) {
     compressed = new uint8_t[size];
     compressed_size = size;
 
-    this->id = id/1000; //TODO: Why divide?
+    this->id = id;  //TODO: Why divide by 1000 ?
 
     // We know where this chunk starts, but not where it ends
     this->start = id;
     this->end = INVALID_TIMESTAMP;
 
     if (!memcpy(compressed, data, size)) {
-        BOOST_LOG_TRIVIAL(fatal) << "Could not allocate " << size << " while parsing chunkVector.";
+        spdlog::critical( "Could not allocate {}  bytes while parsing chunkVector.", size);
+
     }
 }
 
@@ -46,7 +45,7 @@ size_t
 Chunk::getMetricNames(std::vector<std::string> & metricNames) {
 
     for (auto &m : metrics) {
-        metricNames.emplace_back(m->name);
+        metricNames.emplace_back(m->getName());
     }
     return metricNames.size();
 }
@@ -61,7 +60,6 @@ fullname (const char *key, std::vector<std::string> & docs) {
     return s.str();
 }
 
-
 //
 // From StackOverflow https://stackoverflow.com/questions/4901842/in-memory-decompression-with-zlib
 //
@@ -121,9 +119,10 @@ Chunk::ReadVariableSizedInts() {
     const uint8_t *p = this->decompressed; // data;
     auto docSize = *(uint32_t*) this->decompressed;
 
-    if (metrics.size() != metricsInChunk) {
-        BOOST_LOG_TRIVIAL(fatal) << "Chunks in document and in values do not match " << metrics.size() << " != " << metricsInChunk;
-        return -1;
+    auto metricsSize = metrics.size();
+    if (metricsSize != metricsInChunk) {
+        spdlog::debug(  "Different metrics counts in chunk {} != {}",  metrics.size(), metricsInChunk);
+        //return -1;
     }
 
     auto offset = docSize + 8;
@@ -131,7 +130,7 @@ Chunk::ReadVariableSizedInts() {
 
     auto dataRangeSize = this->decompressed_size-offset;
 
-    if (dataRangeSize <= 0) return 0;
+    //if (dataRangeSize <= 0) return 0;
 
     ConstDataRangeCursor dataRangeCursor(q,dataRangeSize);
 
@@ -139,7 +138,7 @@ Chunk::ReadVariableSizedInts() {
 
     for (int m=0; m < metrics.size(); ++m) {
 
-        auto values = metrics[m]->values; //
+        auto values = metrics[m]->getValues(); //
         auto value = values[0];
 
         for (int s = 0; s < deltasInChunk; ++s) {
@@ -158,6 +157,7 @@ Chunk::ReadVariableSizedInts() {
             value += delta;
             values[s + 1] = value;
         }
+        metrics[m]->setSampleCount(deltasInChunk);
     }
     return count;
 }
@@ -175,7 +175,7 @@ typedef struct {
     std::vector<std::string> name_stack;
     std::vector<ChunkMetric *> *metrics;
     bool logNames;
-} visitResults ;
+} visitResults;
 
 static bool
 visit_before (const bson_iter_t *iter, const char *key, void *data)
@@ -219,7 +219,7 @@ visit_int32(const bson_iter_t *, const char *key, int32_t v_int32, void *data) {
     v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack),  BSON_TYPE_INT32, v_int32));
 
     if (v->logNames)
-       BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << v_int32;
+       spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), v_int32);
     return false;
 }
 
@@ -228,7 +228,7 @@ visit_int64(const bson_iter_t *, const char *key, int64_t v_int64, void *data) {
     auto *v = (visitResults *) data;
     v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack),  BSON_TYPE_INT64, v_int64));
     if (v->logNames)
-       BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << v_int64;
+       spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), v_int64);
     return false;
 }
 
@@ -237,7 +237,7 @@ visit_bool(const bson_iter_t *, const char *key, bool v_bool, void *data) {
     auto *v = (visitResults *) data;
     v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack),  BSON_TYPE_BOOL, v_bool));
     if (v->logNames)
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << v_bool;
+        spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), v_bool);
     return false;
 }
 
@@ -246,7 +246,7 @@ visit_double(const bson_iter_t *, const char *key, double v_double, void *data){
     auto *v = (visitResults *) data;
     v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack),  BSON_TYPE_DOUBLE, v_double));
     if (v->logNames)
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << v_double;
+        spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), v_double);
     return false;
 }
 
@@ -259,8 +259,8 @@ visit_timestamp(const bson_iter_t *, const char *key, uint32_t t1, uint32_t t2,
     v->metrics->emplace_back( new ChunkMetric(fullname( s2.c_str(), v->name_stack),  BSON_TYPE_INT32, t2));
 
     if (v->logNames) {
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << t1;
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << t2;
+        spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), t1);
+        spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), t2);
     }
     return false;
 }
@@ -270,7 +270,7 @@ visit_date_time(const bson_iter_t *, const char *key, int64_t v_datetime, void *
     auto *v = (visitResults *) data;
     v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack),  BSON_TYPE_DATE_TIME, v_datetime));
     if (v->logNames)
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << v_datetime;
+        spdlog::debug("Metric: {} = {} ", fullname(key, v->name_stack), v_datetime);
     return false;
 }
 
@@ -278,23 +278,28 @@ static bool
 visit_utf8(const bson_iter_t *, const char *key, size_t, const char *s, void *data){
     auto *v = (visitResults *) data;
     if (v->logNames)
-        BOOST_LOG_TRIVIAL(debug) << "Metric: " << fullname(key, v->name_stack) << " = " << s;
-    return false;
-}
-
-//TODO:: Add?
-static bool
-visit_oid(const bson_iter_t *, const char *, const bson_oid_t *, void *data){
+        spdlog::debug("Metric: {} = {} ", fullname(key, v->name_stack),  s);
     return false;
 }
 
 static bool
-visit_null(const bson_iter_t *, const char *, void *data){
+visit_oid(const bson_iter_t *, const char *key, const bson_oid_t *v_oid, void *data){
+    auto *v = (visitResults *) data;
+    v->metrics->emplace_back( new ChunkMetric(fullname(key, v->name_stack), BSON_TYPE_INT64, (int64_t) v_oid));
+    if (v->logNames)
+        spdlog::debug("Metric: {} = {}", fullname(key, v->name_stack), (int64_t)v_oid);
     return false;
 }
 
 static bool
-visit_binary(const bson_iter_t *, const char *, bson_subtype_t subtype, size_t , const uint8_t *, void *data){
+visit_null(const bson_iter_t *, const char *key, void *data){
+    spdlog::error("Discarding NULL {}", key);
+    return false;
+}
+
+static bool
+visit_binary(const bson_iter_t *, const char *key, bson_subtype_t subtype, size_t , const uint8_t *, void *data){
+    spdlog::error("Discarding binary {}", key);
     return false;
 }
 
@@ -311,14 +316,14 @@ Chunk::ConstructMetrics(const uint8_t* data ) {
     deltasInChunk = *(uint32_t*)(pp + 4);
 
     // This is the number of structures needed
-    BOOST_LOG_TRIVIAL(debug) << "Metrics in chunk to reserve: " << metricsInChunk ;
+    spdlog::debug( "Metrics in chunk to reserve: {}", metricsInChunk);
     metrics.reserve(metricsInChunk);
 
     if ((bsonStruct = bson_new_from_data (data, (size_t)(*dataSize)))) {
         if (bson_iter_init (&iter, bsonStruct)) {
 
             bson_visitor_t vt{};
-            visitResults v{};
+            visitResults results{};
 
             vt.visit_before = visit_before;
             vt.visit_array = visit_array;
@@ -329,17 +334,25 @@ Chunk::ConstructMetrics(const uint8_t* data ) {
             vt.visit_double = visit_double;
             vt.visit_date_time = visit_date_time;
             vt.visit_timestamp = visit_timestamp;
+
+            vt.visit_oid = visit_oid;
+            vt.visit_binary = visit_binary;
+            vt.visit_null = visit_null;
+
             // Only for display
             vt.visit_utf8 = visit_utf8;
 
-            v.visit_table = &vt;
-            v.metrics = &metrics;
+            //results.logNames = true;
+            //spdlog::set_level(spdlog::level::debug);
 
-            ::bson_iter_visit_all(&iter, &vt, &v);
+            results.visit_table = &vt;
+            results.metrics = &metrics;
 
-            //   BOOST_LOG_TRIVIAL(debug) << "Visited: " << v.visited << "  Metrics: " << v.metrics.size();
+            ::bson_iter_visit_all(&iter, &vt, &results);
+
+            spdlog::debug("Visited: {} Metrics: {}", results.visited, results.metrics->size());
             if (metricsInChunk != metrics.size())
-                BOOST_LOG_TRIVIAL(error) << "Wrong number of metrics: " << v.metrics->size() << "!=" << metricsInChunk;
+                spdlog::debug("Different metric names counts: {} != {}", results.metrics->size(), metricsInChunk);
         }
     }
 
@@ -352,8 +365,8 @@ Chunk::ConstructMetrics(const uint8_t* data ) {
 
 void
 Chunk::setTimestampLimits() {
-    start = metrics[0]->values[0];
-    end = metrics[0]->values[getSamplesCount()-1];
+    start = metrics[0]->getValue(0);
+    end = metrics[0]->getValue( getSamplesCount()-1);
 }
 
 bool Chunk::Consume() {
@@ -388,30 +401,30 @@ Chunk::getJsonAtPosition(size_t position) {
     bson_iter_t iter;
     bson_iter_t baz;
 
-    auto newBson = bson_copy(bsonStruct);
+    bson_t* newBson = bson_copy(bsonStruct);
 
     if (bson_iter_init (&iter, newBson)) {
 
         for (int i=0;i<metrics.size(); ++i) {
-            if (bson_iter_find_descendant(&iter, metrics[i]->name.c_str(), &baz)) {
+            if (bson_iter_find_descendant(&iter, metrics[i]->getName().c_str(), &baz)) {
 
                 auto t = bson_iter_type(&baz);
 
                 switch (t) {
                     case BSON_TYPE_INT64:
-                        bson_iter_overwrite_int64(&baz, metrics[i]->values[position]);
+                        bson_iter_overwrite_int64(&baz, metrics[i]->getValue(position));
                         break;
                     case BSON_TYPE_INT32:
-                        bson_iter_overwrite_int32(&baz, metrics[i]->values[position]);
+                        bson_iter_overwrite_int32(&baz, metrics[i]->getValue( position ));
                         break;
                     case BSON_TYPE_DATE_TIME:
-                        bson_iter_overwrite_date_time(&baz, metrics[i]->values[position]);
+                        bson_iter_overwrite_date_time(&baz, metrics[i]->getValue( position));
                         break;
                     case BSON_TYPE_DOUBLE:
-                        bson_iter_overwrite_double(&baz, metrics[i]->values[position]);
+                        bson_iter_overwrite_double(&baz, metrics[i]->getValue( position));
                         break;
                     case BSON_TYPE_BOOL:
-                        bson_iter_overwrite_bool(&baz, metrics[i]->values[position]);
+                        bson_iter_overwrite_bool(&baz, metrics[i]->getValue( position));
                         break;
                     case BSON_TYPE_UTF8:
                     case  BSON_TYPE_EOD:
@@ -448,7 +461,7 @@ std::string
 Chunk::getJsonFromTimestamp(Timestamp ts) {
 
     for (size_t i=0;i<metrics.size(); ++i) {
-        if (metrics[0]->values[i] == ts) {
+        if (metrics[0]->getValue(i) == ts) {
             return getJsonAtPosition(i);
         }
     }
@@ -456,4 +469,39 @@ Chunk::getJsonFromTimestamp(Timestamp ts) {
     return "{}";
 }
 
+std::string
+Chunk::getCsvAtPosition(size_t pos) {
+
+    std::string ret;
+    for (auto m : metrics) {
+        ret += std::to_string(m->getValue(pos));
+        ret += ",";
+    }
+    return ret;
+}
+
+size_t Chunk::getRawValuesAtPosition(uint64_t *values, size_t pos) {
+    size_t count = 0;
+    for (auto m : metrics)
+        values[count++] = m->getValue(pos);
+
+    return count;
+}
+
+int Chunk::AddMetric(std::string metricName, std::vector<uint64_t> *data)   {
+    ChunkMetric *chunkMetric = new ChunkMetric(metricName);
+
+    auto metricValues = chunkMetric->getValues();
+
+    assert(data->size() <= ChunkMetric::MAX_SAMPLES);
+
+    for (auto & v : *data) *metricValues++ = v;
+    chunkMetric->setSampleCount(data->size());
+
+    metrics.emplace_back(chunkMetric);
+
+    ++metricsInChunk;
+
+    return 0;
+}
 
