@@ -12,9 +12,9 @@ class CSVWriter {
 
 public:
 
-
+    static  size_t OutputMultipleFiles() { return 0; };
+    static  size_t OutputSingleFile() { return 0; };
     size_t dumpCSVTimestamps(Dataset *pDataset, std::string outputPath, Timestamp start, Timestamp end, bool rated);
 };
 
 #endif //FTDCPARSER_CSVWRITER_H
-
