@@ -7,7 +7,9 @@
 
 #include <string>
 #include <vector>
-#include <bson/bson.h>
+#include <bson.h>
+
+ 
 
 class ChunkMetric {
 
@@ -17,11 +19,19 @@ public:
     ChunkMetric(std::string name);
 
     static const int MAX_SAMPLES = 300;
-public:
-    std::string name;
-    bson_type_t  type;
-    uint64_t  values[MAX_SAMPLES];  // All metrics fit in a 64 bit integer so....
 
+    int getSampleCount() { return nSamples; }
+    std::string getName() { return name; }
+    uint64_t *getValues() { return values; }
+    uint64_t  getValue(unsigned long n) { return values[n]; }
+
+    void setSampleCount(int samples) { nSamples = samples; }
+
+private:
+    int nSamples{};
+    std::string name;
+    bson_type_t  type{};
+    uint64_t  values[MAX_SAMPLES];  // All metrics fit in a 64 bit integer so....
 };
 
 
