@@ -9,10 +9,6 @@
 #include <cstddef>
 #include "Dataset.h"
 
-#include <boost/log/core.hpp>
-#include <boost/log/trivial.hpp>
-#include <boost/log/expressions.hpp>
-
 
 class JSONWriter {
 
