@@ -5,8 +5,6 @@
 #ifndef PYFTDC_WRITERTASKLIST_H
 #define PYFTDC_WRITERTASKLIST_H
 
-
-#include <boost/thread/mutex.hpp>
 #include <deque>
 #include "WriterTask.h"
 #include "Timestamp.h"
@@ -21,7 +19,7 @@ public:
     bool isEmpty() { return taskList.empty(); }
 
 private:
-    boost::mutex mu;
+    std::mutex mu;
     ftdcparser::Timestamp start;
     ftdcparser::Timestamp end;
 
